LIBRARY d3d12
EXPORTS
D3D12CoreCreateLayeredDevice=d3d12_ms.D3D12CoreCreateLayeredDevice @104
D3D12CoreGetLayeredDeviceSize=d3d12_ms.D3D12CoreGetLayeredDeviceSize @105
D3D12CoreRegisterLayers=d3d12_ms.D3D12CoreRegisterLayers @106
D3D12CreateDevice=gfxrecon_D3D12CreateDevice @101
D3D12CreateRootSignatureDeserializer=gfxrecon_D3D12CreateRootSignatureDeserializer @107
D3D12CreateVersionedRootSignatureDeserializer=gfxrecon_D3D12CreateVersionedRootSignatureDeserializer @108
D3D12DeviceRemovedExtendedData=d3d12_ms.D3D12DeviceRemovedExtendedData @109
D3D12EnableExperimentalFeatures=gfxrecon_D3D12EnableExperimentalFeatures @110
D3D12GetDebugInterface=gfxrecon_D3D12GetDebugInterface @102
D3D12PIXEventsReplaceBlock=d3d12_ms.D3D12PIXEventsReplaceBlock @111
D3D12PIXGetThreadInfo=d3d12_ms.D3D12PIXGetThreadInfo @112
D3D12PIXNotifyWakeFromFenceSignal=d3d12_ms.D3D12PIXNotifyWakeFromFenceSignal @113
D3D12PIXReportCounter=d3d12_ms.D3D12PIXReportCounter @114
D3D12SerializeRootSignature=gfxrecon_D3D12SerializeRootSignature @115
D3D12SerializeVersionedRootSignature=gfxrecon_D3D12SerializeVersionedRootSignature @116
D3D12GetInterface=gfxrecon_D3D12GetInterface @117
D3D12CreateVersionedRootSignatureDeserializerFromSubobjectInLibrary=gfxrecon_D3D12CreateVersionedRootSignatureDeserializerFromSubobjectInLibrary @118
GetBehaviorValue=d3d12_ms.GetBehaviorValue @100
SetAppCompatStringPointer=d3d12_ms.SetAppCompatStringPointer @103
