# CppHeaderParser package
# Author: Jashua Cloutier (contact via sourceforge username:senexcanis)

from .CppHeaderParser import *
from .CppHeaderParser import __version__

# __all__ = ['CppHeaderParser']
